@@ -8,6 +8,8 @@
 
 import time
 import math
+import gameclock.i18n
+import gettext
 
 class Clock:
     """The main clock engine
@@ -21,24 +23,21 @@ class Clock:
     observations (watching the clocks compared to the Gnome clock
     applet)
     """
-    last_start = 0 # 0 if stopped
-    text = "" # rendered version of the clock
-    dead = False
-    game = None
-    # a clock is part of a chained list
-    next = None
-    # usually, clocks go backwards, but some games might require it to go other ways
-    factor = -1
 
-    def __init__(self, game, delay = None):
+    def __init__(self, **settings):
         """Setup the clock engine
         
         The clock is stopped by default and the display is refreshed
         """
-        self.time = game.default_time
-        self.dead = self.time <= 0 # calculated only after update() and should be used only this way, it is a cache
-        self.game = game
-        self.update()
+        # the time, in miliseconds
+        self.time = settings['start_time']
+        # 0 if stopped
+        self.last_start = 0
+        # usually, clocks go backwards, but some games might require it to go other ways
+        self.factor = -1
+        # a clock is part of a chained list
+        self.next = settings['next_clock'] if 'next_clock' in settings else None
+        self.moves = 0
 
     def start(self):
         """Start the timer
@@ -46,7 +45,6 @@ class Clock:
         This marks the new timestamp and sets the background color
         """
         self.last_start = time.time()
-        # no need to update, time shouldn't have changed since last stop
 
     def stop(self):
         """Stop the timer
@@ -68,7 +66,7 @@ class Clock:
         if self.last_start:
             self.time = self.get_time()
             self.last_start = 0
-        self.update()
+            self.moves += 1
 
     def pause(self):
         """pause/unpause the timer
@@ -76,9 +74,7 @@ class Clock:
         this will start the timeer if stopped and stop it if started
         """
         if self.last_start:
-            self.time = self.get_time()
-            self.last_start = 0          
-            self.update()
+            self.stop()
         else:
             self.start()
 
@@ -93,29 +89,40 @@ class Clock:
             diff = 0
         return self.time + (self.factor * diff*1000)
 
+    def moves_fmt(self):
+        return ngettext("%d move", "%d moves", self.moves) % self.moves
+
+    def is_dead(self):
+        return self.get_time() <= 0
+
     def update(self):
         """Refresh the display of the clock's widget"""
-        # convert to seconds because that's what gmtime wants
-        miliseconds = self.get_time()
-        secs = miliseconds/1000
-        self.text = time.strftime("%M:%S", time.gmtime(abs(secs)))
-        days = abs(secs) / ( 24 * 60 * 60 )
-        if days >= 1:
-            self.text = "%dd " % days + self.text
-        self.dead = secs <= 0
-        if self.dead:
-            self.text = "-" + self.text
-        if self.game.miliseconds:
-            mantissa = ".%03d" % (abs(miliseconds) - (math.floor(abs(secs))*1000))
-            self.text += mantissa[:2]
-        return self.text
+        return self.format()
+
+    suggested_formats = [ '%02i:%02d', '%02i:%04.1f', '%i:%02i:%02d', '%i:%02i:%04.1f' ]
+
+    def format(self, fmt = '%02i:%02d'):
+        """Format this clock's internal time as a human-readable string.
+
+Can contain 2 or three formatters. If there are only two, hours are displayed in the minutes."""
+        miliseconds = abs(self.get_time())
+        if self.get_time() < 0:
+            fmt = '-' + fmt
+        if fmt.count('%') == 3:
+            hours, milliseconds = divmod(miliseconds, 3600000)
+        minutes, milliseconds = divmod(miliseconds, 60000)
+        seconds = float(milliseconds) / 1000
+        if fmt.count('%') == 3:
+            return fmt % (hours, minutes, seconds)
+        else:
+            return fmt % (minutes, seconds)
 
     def __str__(self):
         """make a better string representation of the objects
         
         we basically dump all variables and some functions
         """
-        return "  clock engine %s time: %d last: %d diff: %f dead: %d text: %s\n  next %s" % ( object.__str__(self), self.time, self.last_start, time.time() - self.last_start, self.dead, self.text, self.next)
+        return "  clock engine %s time: %d last: %d diff: %f dead: %d text: %s\n  next %s" % ( object.__str__(self), self.time, self.last_start, time.time() - self.last_start, self.is_dead(), self.format(), self.next)
 
 
 class ChessClock(Clock):
@@ -129,20 +136,21 @@ A typical setting is 5 minutes each, which is considered to be a "blitz". 2 minu
     pass
 
 
-class FisherClock(Clock):
-    """A chess clock, as modified by Garry Fisher
+class FischerChessClock(Clock):
+    """A chess clock, as modified by Garry Fischer
 
 This is a regular chess clock with one little twist: every time a player finishes its turn, he gets extra time. This allows for timed game that are still fairly interactive, as the player is forced to move within a certain timeframe.
 
 A typical setting is 2 minutes + 10 seconds delay, which leads to games of around 10 to 20 minutes."""
     delay = 10
 
-    def __init__(self, game, delay):
-        Clock.__init__(self, game, delay)
-        self.delay = delay
+    def __init__(self, **settings):
+        self.delay = settings['delay']
+        del settings['delay']
+        Clock.__init__(self, **settings)
     
     def stop(self):
-        """end the turn, fisher style
+        """end the turn, fischer style
 
         this increments the current clock before switching turns as normal
         """
@@ -157,15 +165,77 @@ A player gets a specific amount of time to play his turn, but the leftover time
     # we need to remember the original time
     default_time = None
 
-    def __init__(self, game, delay = None):
-        Clock.__init__(self, game, delay)
-        self.default_time = self.time
+    def __init__(self, **settings):
+        Clock.__init__(self, **settings)
+        self.default_time = settings['start_time']
 
     def stop(self):
         """override the end_turn function to reset the timers at the end of turns"""
         Clock.stop(self)
         self.time = self.default_time
-        self.update()
+
+class GoStandardByoyomiClock(Clock):
+    """a regular go clock.
+
+    it has a certain time where it behaves like a regular chess clock,
+    but then behaves like a board clock after the given time. at that
+    point there is a certain number of "byoyomis" of a chosen "delay"
+    that moves need to be played within."""
+
+    def __init__(self, **settings):
+        self.delay = settings['delay']
+        # number of extra time periods, if 0, no time periods are left
+        # and the user dies
+        # we increment this because we burn the first byoyomi when entering it
+        self._byoyomi = settings['byoyomi'] + 1
+        self._start_byoyomi = settings['byoyomi']
+        Clock.__init__(self, **settings)
+
+    def get_byoyomi(self):
+        """look if we are dead and consume a byoyomi if so
+
+should be called as often as necessary to display things
+consistently. returns the number of byoyomis used (-1 being regular
+game play, 0 being none, 1 being one, etc).
+"""
+        # can't use our own is_dead, it's subverted by the byoyomi logic
+        if Clock.is_dead(self) and not self.is_dead():
+            self._byoyomi -= 1
+            if self.is_dead():
+                self.time = 0
+            else:
+                # reset the timer only if we didn't blow byoyomi
+                self.time = self.delay
+            if self.running():
+                self.start() # reset
+        return self._byoyomi
+
+    def set_byoyomi(self, byoyomi):
+        self._start_byoyomi = byoyomi
+        self._byoyomi = byoyomi + 1
+
+    def is_dead(self):
+        return self._byoyomi <= 0
+
+    def is_byoyomi(self):
+        return self.get_byoyomi() <= self._start_byoyomi
+
+    def format(self, fmt = '%02i:%02d'):
+        self.get_byoyomi()
+        return Clock.format(self, fmt)
+
+    def stop(self):
+        Clock.stop(self)
+        if self.is_byoyomi() and not self.is_dead():
+            self.time = self.delay
+
+    def moves_fmt(self):
+        ret = Clock.moves_fmt(self)
+        if self.is_byoyomi():
+            ret += ngettext(' %d byoyomi', ' %d byoyomis', self._byoyomi) % self._byoyomi
+        if self.is_dead():
+            ret += _(' (lost)')
+        return ret
 
 class HourglassClock(Clock):
     """Hourglass emulation.
@@ -176,12 +246,12 @@ We do allow each player to start with a certain amonut of time (or "sand", if yo
 
 Note that this doesn't make much sense with more than two players..."""
 
-    def __init__(self, game, delay = None):
+    def __init__(self, **settings):
         """this overrides the base constructor to make only one clock have an initial time
 
         basically, this is to represent that when you start it, the hour glass is empty on one side
         """
-        Clock.__init__(self, game, delay)
+        Clock.__init__(self, **settings)
 
     def start(self):
         """reimplement the start() function altogether
